# Generated by Django 4.2.23 on 2025-11-06 10:26

from django.db import (
    migrations,
    models,
)
from django.db.utils import (
    DEFAULT_DB_ALIAS,
)

from edu_rdm_integration.stages.collect_data.models import (
    RDMCollectingDataCommandProgress,
    RDMCollectingDataStage,
    RDMCollectingDataSubStage,
)


def copy_task_manager_function(apps, schema_editor):
    """Копирование данных полей task, manager и function в новые поля не являющиеся FK."""

    # Проверяем, что это применяется к основной БД
    if schema_editor.connection.alias == DEFAULT_DB_ALIAS:
        with schema_editor.connection.cursor() as cursor:
            cursor.execute(
                f"""
                UPDATE {RDMCollectingDataCommandProgress._meta.db_table}
                SET temp_task_id=task_id;

                UPDATE {RDMCollectingDataSubStage._meta.db_table}
                SET temp_function_id=function_id;

                UPDATE {RDMCollectingDataStage._meta.db_table}
                SET temp_manager_id=manager_id;
                """
            )


class Migration(migrations.Migration):
    dependencies = [
        ('edu_rdm_integration_collect_data_stage', '0007_rdmcollectingdatacommandprogress_temp_task_id_and_more'),
    ]

    operations = [
        migrations.RunPython(copy_task_manager_function, reverse_code=migrations.RunPython.noop),
    ]



from django.db.models import (
    Case,
    CharField,
    F,
    Manager,
    PositiveSmallIntegerField,
    Q,
    Value,
    When,
)
from django.db.models.functions import (
    Cast,
)

from edu_rdm_integration.stages.upload_data.uploader_log.enums import (
    RequestResultStatus,
)


class UploaderClientLogManager(Manager):
    """Менеджер модели журнала Загрузчика данных в витрину."""

    def get_queryset(self):
        """Возвращает кварисет."""
        query = super().get_queryset()

        result_status = Case(
            When(
                Q(Q(error__isnull=True) | Q(error__exact='')) & Q(Q(response__isnull=False) & ~Q(response__exact='')),
                then=Value(RequestResultStatus.SUCCESS),
            ),
            default=Value(RequestResultStatus.ERROR),
            output_field=PositiveSmallIntegerField(),
        )

        query = query.annotate(
            request_datetime=F('date_time'),
            result_status=result_status,
        )

        return query



# Generated by Django 4.2.23 on 2025-11-05 16:17

from django.db import (
    migrations,
    models,
)


class Migration(migrations.Migration):
    dependencies = [
        ('edu_rdm_integration_upload_data_stage', '0004_fix_fk_constraints'),
    ]

    operations = [
        migrations.AddField(
            model_name='rdmexportingdatasubstageuploaderclientlog',
            name='temp_entry_id',
            field=models.BigIntegerField(null=True, verbose_name='Лог запроса и ответа'),
        ),
        migrations.AddField(
            model_name='rdmuploadstatusrequestlog',
            name='temp_entry_id',
            field=models.BigIntegerField(null=True, verbose_name='Cвязь запроса статуса с запросом в витрину'),
        ),
    ]



# Generated by Django 4.2.23 on 2025-11-07 18:23

from django.db import (
    migrations,
    models,
)


class Migration(migrations.Migration):
    dependencies = [
        (
            'edu_rdm_integration_upload_data_stage',
            '0006_alter_rdmexportingdatasubstageuploaderclientlog_entry_and_more',
        ),
    ]

    operations = [
        migrations.RemoveField(
            model_name='rdmexportingdatasubstageuploaderclientlog',
            name='entry_id',
        ),
        migrations.RemoveField(
            model_name='rdmuploadstatusrequestlog',
            name='entry_id',
        ),
    ]



# Generated by Django 4.2.23 on 2025-11-05 16:17

from django.db import (
    migrations,
    models,
)


class Migration(migrations.Migration):
    dependencies = [
        ('edu_rdm_integration_collect_data_stage', '0006_fix_fk_constraints'),
    ]

    operations = [
        migrations.AddField(
            model_name='rdmcollectingdatacommandprogress',
            name='temp_task_id',
            field=models.UUIDField(blank=True, null=True, verbose_name='Асинхронная задача'),
        ),
        migrations.AddField(
            model_name='rdmcollectingdatastage',
            name='temp_manager_id',
            field=models.UUIDField(blank=True, null=True, verbose_name='Менеджер ранера Функции'),
        ),
        migrations.AddField(
            model_name='rdmcollectingdatasubstage',
            name='temp_function_id',
            field=models.UUIDField(null=True, verbose_name='Функция'),
        ),
    ]



# Generated by Django 4.2.23 on 2025-11-07 18:23

from django.db import (
    migrations,
    models,
)


class Migration(migrations.Migration):
    dependencies = [
        ('edu_rdm_integration_collect_data_stage', '0008_alter_rdmcollectingdatacommandprogress_task_and_more'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='rdmcollectingdatacommandprogress',
            name='task_id',
        ),
        migrations.RemoveField(
            model_name='rdmcollectingdatastage',
            name='manager_id',
        ),
        migrations.RemoveField(
            model_name='rdmcollectingdatasubstage',
            name='function_id',
        ),
    ]



# Generated by Django 4.2.23 on 2025-11-06 10:26

from django.db import (
    migrations,
    models,
)
from django.db.utils import (
    DEFAULT_DB_ALIAS,
)

from edu_rdm_integration.stages.upload_data.models import (
    RDMExportingDataSubStageUploaderClientLog,
    RDMUploadStatusRequestLog,
)


def copy_entry(apps, schema_editor):
    """Копирование данных из поля entry_id во временное поле temp_entry_id."""

    # Проверяем, что применяем для основной БД
    if schema_editor.connection.alias == DEFAULT_DB_ALIAS:
        with schema_editor.connection.cursor() as cursor:
            cursor.execute(
                f"""
                UPDATE {RDMExportingDataSubStageUploaderClientLog._meta.db_table}
                SET temp_entry_id=entry_id;

                UPDATE {RDMUploadStatusRequestLog._meta.db_table}
                SET temp_entry_id=entry_id;
                """
            )


class Migration(migrations.Migration):
    dependencies = [
        (
            'edu_rdm_integration_upload_data_stage',
            '0005_rdmexportingdatasubstageuploaderclientlog_temp_entry_id_and_more',
        ),
    ]

    operations = [
        migrations.RunPython(copy_entry, reverse_code=migrations.RunPython.noop),
    ]



# Generated by Django 4.2.23 on 2025-11-10 11:53

from django.db import (
    migrations,
)


class Migration(migrations.Migration):
    dependencies = [
        ('edu_rdm_integration_collect_data_stage', '0009_remove_rdmcollectingdatacommandprogress_task_id_and_more'),
    ]

    operations = [
        migrations.RenameField(
            model_name='rdmcollectingdatacommandprogress',
            old_name='temp_task_id',
            new_name='task_id',
        ),
        migrations.RenameField(
            model_name='rdmcollectingdatastage',
            old_name='temp_manager_id',
            new_name='manager_id',
        ),
        migrations.RenameField(
            model_name='rdmcollectingdatasubstage',
            old_name='temp_function_id',
            new_name='function_id',
        ),
    ]



from django.apps import (
    apps,
)
from django.conf import (
    settings,
)
from django.db.utils import (
    DEFAULT_DB_ALIAS,
)


app_configs = apps.get_app_configs()


class RdmRouter:
    """Роутер, направляющий все запросы к моделям из приложения 'edu_rdm_integration' в базу данных 'rdm_db'."""

    db_name = 'rdm'
    # Имена моделей, для исключение из БД РВД подключенных приложений.
    exclude_rdm_db_model_names = set()

    def __init__(self):
        # Получаем путь через AppConfig одного из приложений в пакете
        try:
            package_app_config = apps.get_app_config('edu_rdm_integration')
            base_package_path = package_app_config.path
        except LookupError:
            # Обработка случая, если приложение 'edu_rdm_integration' не найдено
            base_package_path = None

        self.use_rdm_db = getattr(settings, 'USE_RDM_DB', False)
        self.edu_rdm_apps = self._get_edu_rdm_apps(base_package_path)
        self.exclude_rdm_db_model_names = {model_name.lower() for model_name in self.exclude_rdm_db_model_names}

    def _get_edu_rdm_apps(self, base_package_path):
        """Динамически определяет приложения, которые находятся в пакете 'edu_rdm_integration'."""
        app_labels = []
        for app_config in apps.get_app_configs():
            if app_config.path.startswith(base_package_path):
                app_labels.append(app_config.label)

        # добавляем таблицу uploader_client_entry
        app_labels.append('uploader_client')

        return app_labels

    def is_rdm_app(self, app_label: str, model_name: str) -> bool:
        """Проверяет, что модель и приложение относится к РВД.

        Args:
            app_label: Название приложения
            model_name: Название модели
        Returns: Относится ли приложение и модель к РВД
        """
        return app_label in self.edu_rdm_apps and model_name not in self.exclude_rdm_db_model_names

    def db_for_read(self, model, **hints):
        return self.db_for_write(model, **hints)

    def db_for_write(self, model, **hints):
        res = None
        if self.is_rdm_app(model._meta.app_label, model.__name__.lower()) and self.use_rdm_db:
            res = self.db_name

        return res

    def allow_relation(self, obj1, obj2, **hints):
        # Если оба объекта принадлежат 'edu_rdm_integration', то отношения между ними разрешены.
        is_rdm_app1 = self.is_rdm_app(obj1._meta.app_label, obj1._meta.model.__name__.lower())
        is_rdm_app2 = self.is_rdm_app(obj2._meta.app_label, obj2._meta.model.__name__.lower())
        is_allow = False

        # Разрешить отношения, если оба объекта из RDM-приложений.
        if is_rdm_app1 and is_rdm_app2:
            is_allow = True
        # Если ни один из объектов не принадлежит RDM-приложениям, обработка делегируется следующим роутерам.
        elif not is_rdm_app1 and not is_rdm_app2:
            is_allow = None

        return is_allow

    def allow_migrate(self, db, app_label, model_name=None, **hints):
        model_name = model_name.lower() if model_name else ''
        is_rdm_app = self.is_rdm_app(app_label, model_name)
        is_allow = None

        if is_rdm_app:
            if db == self.db_name and self.use_rdm_db or db == DEFAULT_DB_ALIAS and not self.use_rdm_db:
                is_allow = True
            else:
                is_allow = False
        elif db == self.db_name and not is_rdm_app:
            is_allow = False

        return is_allow
